import wpilib
import wpilib.drive
import math
import ctre

class MyRobot(wpilib.TimedRobot):
    def robotInit(self):
        self.leftFront = ctre.WPI_VictorSPX(0)
        self.leftRear = ctre.WPI_VictorSPX(1)
        self.rightFront = ctre.WPI_VictorSPX(2)
        self.rightRear = ctre.WPI_VictorSPX(3)

        self.left = wpilib.MotorControllerGroup(self.leftFront, self.leftRear)
        self.right = wpilib.MotorControllerGroup(self.rightFront, self.rightRear)
        self.right.setInverted(True)

        self.drivetrain = wpilib.drive.DifferentialDrive(self.left, self.right)

        self.primary = wpilib.XboxController(0)

        self.gyro = wpilib.ADXRS450_Gyro()
        self.gyro.calibrate()

    def autonomousInit(self):
        self.gyro.reset()

    def autonomousPeriodic(self):
        if self.gyro.getAngle() < 90:
            self.drivetrain.tankDrive(.5, -.5)
        else:
            self.drivetrain.tankDrive(0, 0)

    def teleopInit(self):
        pass

    def teleopPeriodic(self):
        self.drivetrain.arcadeDrive(self.primary.getLeftY(), self.primary.getLeftX())

if __name__ == "__main__":
    wpilib.run(MyRobot)